@@ -1,3 +1,4 @@
+import importlib
 import inspect
 import logging
 import sys
@@ -7,11 +8,11 @@ from flask.json import JSONEncoder
 from flask_injector import FlaskInjector, request
 from flask_sqlalchemy import SQLAlchemy
 from injector import singleton as singleton
-from werkzeug.utils import find_modules, import_string
+from werkzeug.utils import import_string
 
 from xflask.common.configuration import Configuration
 from xflask.common.logger import Logger
-from xflask.common.util import get_root_dir, get_file_path, get_xflask_path, import_modules
+from xflask.common.util import get_root_dir, get_file_path, get_xflask_path, scan_namespaces
 from xflask.component import Component
 from xflask.context import ApplicationStateListener
 from xflask.json import JSON_SERIALIZERS
@@ -20,8 +21,6 @@ from xflask.web.error_handler import BasicErrorHandler
 from xflask.web.error_handler import ErrorHandler
 from xflask.web.filter import Filter
 from xflask.web.security.auth_manager import AuthManager
-from xflask.web.security.jwt_auth_filter import JwtAuthFilter
-from xflask.web.security.jwt_auth_manager import JwtAuthManager
 
 
 class Application(object):
@@ -29,10 +28,6 @@ class Application(object):
 
     DEF_LOG_FILE = 'conf/logging.yml'
 
-    DEF_FILTERS = [JwtAuthFilter]
-
-    DER_AUTH_MANAGER = JwtAuthManager
-
     DEF_ERROR_HANDLER = BasicErrorHandler
 
     def __init__(self, db, conf_files=None):
@@ -44,11 +39,13 @@ class Application(object):
 
         self.components = []
 
+        self._component_registry = dict()
+
         self.error_handler = self.DEF_ERROR_HANDLER
 
-        self.filters = self.DEF_FILTERS
+        self.filters = []
 
-        self.auth_manager = self.DER_AUTH_MANAGER
+        self.auth_manager = None
 
         self.json_serializers = []
 
@@ -56,18 +53,29 @@ class Application(object):
 
     #### SETTER ####
 
-    def set_filters(self, filters=[]):
-        self.filters = filters
+    def get_property(self, property_name):
+        return self.conf.get(property_name)
 
     def set_error_handler(self, error_handler):
         self.error_handler = error_handler
 
-    def set_listeners(self, listeners=[]):
-        self.listeners = listeners
-
     def set_auth_manager(self, auth_manager):
         self.auth_manager = auth_manager
 
+    def register_filter(self, filter, order=None):
+        order = order or filter.order
+        if order is None:
+            self.filters.append(filter)
+        else:
+            self.filters.insert(order, filter)
+
+    def register_listener(self, listener, order=None):
+        order = order or listener.order
+        if order is None:
+            self.listeners.append(listener)
+        else:
+            self.listeners.insert(order, listener)
+
     def register_json_serializer(self, serializer):
         self.json_serializers.append(serializer)
 
@@ -77,7 +85,7 @@ class Application(object):
     #### GETTER ####
 
     def get_component(self, clazz):
-        component = self.flask_injector.injector.get(clazz)
+        component = self.injector.injector.get(clazz)
         return None if type(component) == type else component
 
     #### RUN APP ####
@@ -171,12 +179,23 @@ class Application(object):
         self._on_start()
 
     def _init_models(self):
-        # TODO: nested packages
-        import_modules(get_root_dir(), self.conf.get('MODEL') or [])
+        for package in self.conf.get('MODEL') or []:
+            self._logger.debug('>>> Scan models in %s', package)
+
+            namespaces = scan_namespaces(package)
+            for namespace in namespaces:
+                self._logger.debug('Register model: %s', namespace)
+
+                try:
+                    importlib.import_module(namespace)
+                except Exception as e:
+                    self._logger.exception('!!! Failed to initialize model: %s', namespace, e)
+
+                    sys.exit()
 
     def _init_components(self):
 
-        def configure(binder):
+        def static_module(binder):
             # config
             binder.bind(Configuration, self.conf, scope=singleton)
 
@@ -184,77 +203,93 @@ class Application(object):
             if self.conf.exist('SQLALCHEMY_DATABASE_URI'):
                 binder.bind(SQLAlchemy, to=self.db, scope=singleton)
 
-            # components (registered manually)
+            # manually registered components
             for obj in self.components:
                 if issubclass(obj, Component):
-
-                    if issubclass(obj, ApplicationStateListener):
-                        self.listeners.append(obj)
+                    self._component_registry[obj] = ''
 
                     scope = singleton if obj.scope == 'singleton' else request
                     binder.bind(obj, obj, scope)
 
-        def configure_scan(binder):
-            for package in self.conf.get('COMPONENT'):
+        def scan_module(binder):
+            for package in self.conf.get('COMPONENT') or []:
                 try:
-                    self._logger.debug('>>> Scan modules in %s', package)
+                    self._logger.debug('>>> Scan components in %s', package)
 
-                    # TODO: nested packages
-                    for pkg_name in find_modules(package):
+                    namespaces = scan_namespaces(package)
+                    for namespace in namespaces:
                         try:
-                            module = import_string(pkg_name)
+                            module = import_string(namespace)
                             class_names = [m[0] for m in inspect.getmembers(module, inspect.isclass) if
                                            m[1].__module__ == module.__name__]
 
                             valid_module = False
+
                             for class_name in class_names:
                                 obj = getattr(module, class_name)
 
                                 if issubclass(obj, Controller):
                                     continue
 
-                                if issubclass(obj, ApplicationStateListener):
-                                    self._component_listeners.append(obj)
-
                                 if issubclass(obj, Component):
                                     if obj.abstract is True:
                                         continue
 
+                                    self._component_registry[obj] = ''
+
                                     valid_module = True
 
                                     scope = singleton if obj.scope == 'singleton' else request
                                     binder.bind(obj, obj, scope)
 
                                     self._logger.debug('Register component: %s', class_name)
-                                else:
-                                    del module
-                                    self._logger.debug('!!! Invalid component: %s', class_name)
 
                             if not valid_module:
                                 del module
 
                         except Exception as e:
-                            self._logger.exception('!!! Failed to initialize component in %s', pkg_name, e)
+                            self._logger.exception('!!! Failed to initialize modules in %s', namespace, e)
+
                             sys.exit()
 
                 except Exception:
                     self._logger.debug('!!! No modules founded in %s', package)
 
-        self.flask_injector = FlaskInjector(app=self.app, modules=[configure, configure_scan])
+        self.injector = FlaskInjector(app=self.app, modules=[static_module, scan_module])
+
+        # POST INIT
+
+        for component, _ in self._component_registry.items():
+            obj = self.get_component(component)
+
+            if isinstance(obj, ApplicationStateListener):
+                order = obj.order
+                if order is None:
+                    self.listeners.append(obj)
+                else:
+                    self.listeners.insert(order, obj)
+
+            if isinstance(obj, Filter):
+                order = obj.order
+                if order is None:
+                    self.filters.append(obj)
+                else:
+                    self.filters.insert(order, obj)
 
     def _init_controllers(self):
-        for package in self.conf.get('CONTROLLER'):
+        for package in self.conf.get('CONTROLLER') or []:
             try:
-                self._logger.debug('>>> Scan modules in %s', package)
+                self._logger.debug('>>> Scan controllers in %s', package)
 
-                # TODO: nested packages
-                for module_ns in find_modules(package):
+                namespaces = scan_namespaces(package)
+                for namespace in namespaces:
                     try:
-                        module = import_string(module_ns)
+                        module = import_string(namespace)
                         class_names = [m[0] for m in inspect.getmembers(module, inspect.isclass) if
                                        m[1].__module__ == module.__name__]
 
                         valid_module = False
+
                         for class_name in class_names:
                             controller = getattr(module, class_name)
 
@@ -265,20 +300,21 @@ class Application(object):
                                 valid_module = True
 
                                 try:
-                                    controller.register(self.app, self.flask_injector.injector)
-                                except Exception:
-                                    self._logger.exception('Failed to initialize controller: %s', class_name)
+                                    controller.register(self.app, self.injector.injector)
+                                except Exception as e:
+                                    self._logger.exception('Failed to initialize controller: %s', class_name, e)
+
                                     sys.exit()
 
                                 self._logger.debug('Register controller: %s', class_name)
                             else:
-                                self._logger.debug('!!! Invalid controller: %s', class_name)
+                                self._logger.error('!!! Invalid controller: %s', class_name)
 
                         if not valid_module:
                             del module
 
                     except Exception as e:
-                        self._logger.exception('!!! Failed to initialize controller in %s', module_ns, e)
+                        self._logger.exception('!!! Failed to initialize modules in %s', namespace, e)
                         sys.exit()
 
             except Exception:
@@ -304,7 +340,7 @@ class Application(object):
         if isinstance(error_handler, ErrorHandler):
             error_handler.init(self)
         else:
-            self._logger.debug('!!! Invalid error handler: %s', error_handler.__name__)
+            self._logger.error('!!! Invalid error handler: %s', error_handler.__name__)
 
     def _init_filters(self):
         for idx, filter_ in enumerate(self.filters):
@@ -320,7 +356,7 @@ class Application(object):
                 self.app.before_request(filter_.before)
                 self.app.after_request(filter_.after)
             else:
-                self._logger.debug('!!! Invalid filter: %s', filter_.__name__)
+                self._logger.error('!!! Invalid filter: %s', filter_.__name__)
 
     def _init_auth_manager(self):
         if self.auth_manager is None:
@@ -333,7 +369,10 @@ class Application(object):
 
             self.auth_manager = auth_manager
 
-        auth_manager.init(self)
+        if isinstance(auth_manager, AuthManager):
+            auth_manager.init(self)
+        else:
+            self._logger.error('!!! Invalid auth manager: %s', auth_manager.__name__)
 
     def _on_start(self):
         with self.app.app_context():
@@ -347,7 +386,7 @@ class Application(object):
                 if isinstance(listener, ApplicationStateListener):
                     listener.on_start(self)
                 else:
-                    self._logger.debug('!!! Invalid application state listener: %s', listener.__name__)
+                    self._logger.error('!!! Invalid application state listener: %s', listener.__name__)
 
     def _on_stop(self):
         with self.app.app_context():
