@@ -20,11 +20,14 @@ from werkzeug.datastructures import CombinedMultiDict
 from werkzeug.routing import parse_rule
 from wtforms import Form
 from wtforms.form import FormMeta
+from wtforms.fields.core import UnboundField
 
 from xflask.common.util import to_dict, merge_dict
 from xflask.component import Component
 from xflask.exception import Exception as XException
 from xflask.type.sys_code import SysCode
+from xflask.wtforms import Form as XForm
+from xflask.classy.annotation import UrlParam, Header
 
 _py2 = sys.version_info[0] == 2
 
@@ -214,10 +217,13 @@ class FlaskView(object):
             try:
                 req_data = request.get_json() if request.is_json else CombinedMultiDict(
                     (request.files, request.form)).to_dict()
+
+                req_data.update(request.view_args)
             except Exception:
                 req_data = {}
 
             injected_args = {}
+
             for arg_name, arg_annotation in view_args.items():
                 # form class
                 if isinstance(arg_annotation, FormMeta):
@@ -238,6 +244,41 @@ class FlaskView(object):
                         raise XException(SysCode.INVALID, form.errors)
 
                     arg_value = form
+                # form field
+                # elif isinstance(arg_annotation, UnboundField):
+                #     class F(XForm):
+                #         pass
+                #
+                #     setattr(F, arg_name, arg_annotation.field_class(**arg_annotation.kwargs))
+                #
+                #     form = F.from_json(req_data)
+                #     if not form.validate():
+                #         raise XException(SysCode.INVALID, form.errors)
+                #
+                #     arg_value = form[arg_name].data
+                # url parameter
+                elif isinstance(arg_annotation, UrlParam) or issubclass(arg_annotation, UrlParam):
+                    args = request.args
+                    arg_annotation = arg_annotation() if inspect.isclass(arg_annotation) else arg_annotation
+                    arg_name = arg_annotation.name if arg_annotation.name is not None else arg_name
+
+                    arg_value = args.get(arg_name, arg_annotation.default, arg_annotation.type)
+                    if arg_value is None:
+                        arg_value = args.getlist(arg_name, arg_annotation.type)
+
+                    if arg_value is None and arg_annotation.required is True:
+                        error = {arg_name: ['This url parameter is required']}
+                        raise XException(SysCode.INVALID, error)
+                # header
+                elif isinstance(arg_annotation, Header) or issubclass(arg_annotation, Header):
+                    args = request.headers
+                    arg_annotation = arg_annotation() if inspect.isclass(arg_annotation) else arg_annotation
+                    arg_name = arg_annotation.name if arg_annotation.name is not None else arg_name
+
+                    arg_value = args.get(arg_name)
+                    if arg_value is None and arg_annotation.required is True:
+                        error = {arg_name: ['This header is required']}
+                        raise XException(SysCode.INVALID, error)
                 # component
                 elif inspect.isclass(arg_annotation) and issubclass(arg_annotation, Component):
                     arg_value = injector.get(arg_annotation)
