@@ -1,4 +1,5 @@
 from xflask.wtforms import Field
+from xflask.wtforms import ValidationError
 
 class EnumField(Field):
 
@@ -10,7 +11,9 @@ class EnumField(Field):
         if value_list is None or len(value_list) == 0:
             return None
 
-        self.data = self.enum.value_of(value_list[0]) or value_list[0]
+        self.data = self.enum.value_of(value_list[0])
+        if self.data is None:
+            raise ValidationError('This field is invalid')
 
     def _value(self):
         return self.data.code() if self.data is not None else None
